@@ -1,8 +1,10 @@
 # -*- coding: utf-8 -*-
 from Acquisition import aq_base
 from App.config import getConfiguration
+from collective.exportimport import config
 from collective.exportimport.interfaces import IBase64BlobsMarker
 from collective.exportimport.interfaces import IMigrationMarker
+from collective.exportimport.interfaces import IPathBlobsMarker
 from collective.exportimport.interfaces import IRawRichTextMarker
 from operator import itemgetter
 from plone import api
@@ -10,13 +12,10 @@ from plone.i18n.normalizer.interfaces import IIDNormalizer
 from plone.restapi.interfaces import ISerializeToJson
 from Products.Five import BrowserView
 from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
-from zope.component import adapter
 from zope.component import getMultiAdapter
 from zope.component import getUtility
 from zope.i18n import translate
 from zope.interface import alsoProvides
-from zope.interface import implementer
-from zope.interface import Interface
 from zope.interface import noLongerProvides
 from zope.schema import getFields
 
@@ -25,6 +24,7 @@ import logging
 import os
 import pkg_resources
 import six
+import tempfile
 
 try:
     pkg_resources.get_distribution("Products.Archetypes")
@@ -35,6 +35,7 @@ except pkg_resources.DistributionNotFound:
 else:
     from Products.Archetypes.atapi import ReferenceField
     from Products.Archetypes.interfaces import IBaseObject
+
     HAS_AT = True
 
 try:
@@ -46,6 +47,7 @@ except pkg_resources.DistributionNotFound:
 else:
     from plone.dexterity.interfaces import IDexterityContent
     from plone.dexterity.utils import iterSchemata
+
     HAS_DX = True
 
 try:
@@ -57,6 +59,7 @@ except pkg_resources.DistributionNotFound:
 else:
     from z3c.relationfield.interfaces import IRelationChoice
     from z3c.relationfield.interfaces import IRelationList
+
     HAS_RELATIONS = True
 
 
@@ -66,19 +69,19 @@ _marker = object()
 
 # copied from plone.app.contenttypes inplace migration
 LISTING_VIEW_MAPPING = {  # OLD (AT and old DX) : NEW
-    'all_content': 'full_view',
-    'atct_album_view': 'album_view',
-    'atct_topic_view': 'listing_view',
-    'collection_view': 'listing_view',
-    'folder_album_view': 'album_view',
-    'folder_full_view': 'full_view',
-    'folder_listing': 'listing_view',
-    'folder_listing_view': 'listing_view',
-    'folder_summary_view': 'summary_view',
-    'folder_tabular_view': 'tabular_view',
-    'standard_view': 'listing_view',
-    'thumbnail_view': 'album_view',
-    'view': 'listing_view',
+    "all_content": "full_view",
+    "atct_album_view": "album_view",
+    "atct_topic_view": "listing_view",
+    "collection_view": "listing_view",
+    "folder_album_view": "album_view",
+    "folder_full_view": "full_view",
+    "folder_listing": "listing_view",
+    "folder_listing_view": "listing_view",
+    "folder_summary_view": "summary_view",
+    "folder_tabular_view": "tabular_view",
+    "standard_view": "listing_view",
+    "thumbnail_view": "album_view",
+    "view": "listing_view",
 }
 
 
@@ -92,49 +95,148 @@ class ExportContent(BrowserView):
 
     DROP_PATHS = []
 
-    def __call__(self, portal_type=None, include_blobs=False, download_to_server=False, migration=True):
-        self.portal_type = portal_type
+    def __call__(
+        self,
+        portal_type=None,
+        path=None,
+        depth=-1,
+        include_blobs=1,
+        download_to_server=False,
+        migration=True,
+    ):
+        self.portal_type = portal_type or []
+        if isinstance(self.portal_type, str):
+            self.portal_type = [self.portal_type]
         self.migration = migration
+        self.path = path or "/".join(self.context.getPhysicalPath())
 
-        self.fixup_request()
+        self.depth = int(depth)
+        self.depth_options = (
+            ("-1", "unlimited"),
+            ("0", "0"),
+            ("1", "1"),
+            ("2", "2"),
+            ("3", "3"),
+            ("4", "4"),
+            ("5", "5"),
+            ("6", "6"),
+            ("7", "7"),
+            ("8", "8"),
+            ("9", "9"),
+            ("10", "10"),
+        )
+        self.include_blobs = int(include_blobs)
+        self.include_blobs_options = (
+            ("0", "as download urls"),
+            ("1", "as base-64 encoded strings"),
+            ("2", "as blob paths"),
+        )
+
+        self.update()
 
         if not self.request.form.get("form.submitted", False):
             return self.template()
 
         if not self.portal_type:
+            api.portal.show_message(u"Select at least one type to export", self.request)
             return self.template()
 
-        data = self.export_content(include_blobs=include_blobs)
-        number = len(data)
-        filename = '{}.json'.format(self.portal_type)
-        data = json.dumps(data, sort_keys=True, indent=4)
-
-        if download_to_server:
-            cfg = getConfiguration()
-            filepath = os.path.join(cfg.clienthome, filename)
-            with open(filepath, 'w') as f:
-                f.write(data)
-            msg = u"Exported {} {} as {} to {}".format(number, self.portal_type, filename, filepath)
-            logger.info(msg)
-            api.portal.show_message(msg, self.request)
-            self.request.response.redirect(self.request['ACTUAL_URL'])
+        if self.include_blobs == 1:
+            # Add marker-interface to request to use our custom serializers
+            alsoProvides(self.request, IBase64BlobsMarker)
+        elif self.include_blobs == 2:
+            # Add marker interface to export blob paths
+            alsoProvides(self.request, IPathBlobsMarker)
         else:
-            msg = u"Exported {} {}".format(number, self.portal_type)
+            # Use the default plone.restapi serializer,
+            # which gives a download url.
+            pass
+
+        if self.migration:
+            # Add marker-interface to request to use custom serializers
+            alsoProvides(self.request, IMigrationMarker)
+
+        # to get a useful filename...
+        if self.portal_type and len(self.portal_type) == 1:
+            filename = self.portal_type[0]
+        else:
+            filename = self.path.split("/")[-1]
+        filename = "{}.json".format(filename)
+
+        content_generator = self.export_content()
+
+        number = 0
+        if download_to_server:
+            directory = config.CENTRAL_DIRECTORY
+            if directory:
+                if not os.path.exists(directory):
+                    os.makedirs(directory)
+                    logger.info("Created central export/import directory %s", directory)
+            else:
+                cfg = getConfiguration()
+                directory = cfg.clienthome
+            filepath = os.path.join(directory, filename)
+            with open(filepath, "w") as f:
+                for number, datum in enumerate(content_generator, start=1):
+                    if number == 1:
+                        f.write("[")
+                    else:
+                        f.write(",")
+                    json.dump(datum, f, sort_keys=True, indent=4)
+                if number:
+                    f.write("]")
+            msg = u"Exported {} items ({}) as {} to {}".format(
+                number, ", ".join(self.portal_type), filename, filepath
+            )
             logger.info(msg)
             api.portal.show_message(msg, self.request)
-            response = self.request.response
-            response.setHeader("content-type", "application/json")
-            response.setHeader("content-length", len(data))
-            response.setHeader(
-                "content-disposition",
-                'attachment; filename="{0}"'.format(filename),
-            )
-            return response.write(safe_bytes(data))
+
+            if self.include_blobs == 1:
+                # remove marker interface
+                noLongerProvides(self.request, IBase64BlobsMarker)
+            elif self.include_blobs == 2:
+                noLongerProvides(self.request, IPathBlobsMarker)
+            self.request.response.redirect(self.request["ACTUAL_URL"])
+        else:
+            with tempfile.TemporaryFile(mode="w+") as f:
+                for number, datum in enumerate(content_generator, start=1):
+                    if number == 1:
+                        f.write("[")
+                    else:
+                        f.write(",")
+                    json.dump(datum, f, sort_keys=True, indent=4)
+                if number:
+                    f.write("]")
+                msg = u"Exported {} {}".format(number, self.portal_type)
+                logger.info(msg)
+                api.portal.show_message(msg, self.request)
+                response = self.request.response
+                response.setHeader("content-type", "application/json")
+                response.setHeader("content-length", f.tell())
+                response.setHeader(
+                    "content-disposition",
+                    'attachment; filename="{0}"'.format(filename),
+                )
+                if self.include_blobs == 1:
+                    # remove marker interface
+                    noLongerProvides(self.request, IBase64BlobsMarker)
+                elif self.include_blobs == 2:
+                    noLongerProvides(self.request, IPathBlobsMarker)
+                f.seek(0)
+                return response.write(safe_bytes(f.read()))
+
+    def update(self):
+        """Hook to do something before export."""
 
     def build_query(self):
-        query = {"portal_type": self.portal_type, "sort_on": "path"}
+        query = {
+            "portal_type": self.portal_type,
+            "sort_on": "path",
+            "path": {"query": self.path, "depth": self.depth},
+        }
         # custom setting per type
-        query.update(self.QUERY.get(self.portal_type, {}))
+        for portal_type in self.portal_type:
+            query.update(self.QUERY.get(portal_type, {}))
         query = self.update_query(query)
         return query
 
@@ -142,21 +244,11 @@ class ExportContent(BrowserView):
         """Overwrite this if you want more control over which content to export."""
         return query
 
-    def export_content(self, include_blobs=False):
-        data = []
+    def export_content(self):
         query = self.build_query()
         catalog = api.portal.get_tool("portal_catalog")
         brains = catalog.unrestrictedSearchResults(**query)
         logger.info(u"Exporting {} {}".format(len(brains), self.portal_type))
-        self.safe_portal_type = fix_portal_type(self.portal_type)
-
-        if include_blobs:
-            # Add marker-interface to request to use our custom serializers
-            alsoProvides(self.request, IBase64BlobsMarker)
-
-        if self.migration:
-            # Add marker-interface to request to use custom serializers
-            alsoProvides(self.request, IMigrationMarker)
 
         # Override richtext serializer to export links using resolveuid/xxx
         alsoProvides(self.request, IRawRichTextMarker)
@@ -180,8 +272,9 @@ class ExportContent(BrowserView):
             if not obj:
                 continue
             try:
+                self.safe_portal_type = fix_portal_type(obj.portal_type)
                 serializer = getMultiAdapter((obj, self.request), ISerializeToJson)
-                if getattr(aq_base(obj), 'isPrincipiaFolderish', False):
+                if getattr(aq_base(obj), "isPrincipiaFolderish", False):
                     item = serializer(include_items=False)
                 else:
                     item = serializer()
@@ -198,16 +291,10 @@ class ExportContent(BrowserView):
                     logger.info(u"Skipping {}".format(brain.getURL()))
                     continue
 
-                data.append(item)
+                yield item
             except Exception as e:
                 logger.info(u"Error exporting {}: {}".format(obj.absolute_url(), e))
 
-        if include_blobs:
-            # remove marker interface
-            noLongerProvides(self.request, IBase64BlobsMarker)
-
-        return data
-
     def portal_types(self):
         """A list with info on all content types with existing items."""
         catalog = api.portal.get_tool("portal_catalog")
@@ -229,10 +316,6 @@ class ExportContent(BrowserView):
                 )
         return sorted(results, key=itemgetter("title"))
 
-    def fixup_request(self):
-        """Use this to override stuff (e.g. force a specific language in request)."""
-        return
-
     def global_obj_hook(self, obj):
         """Inspect the content item before serialisation data.
         Bad: Changing the content-item is a horrible idea.
@@ -269,11 +352,11 @@ class ExportContent(BrowserView):
 
         """
         # 1. Drop unused data
-        item.pop('@components', None)
-        item.pop('next_item', None)
-        item.pop('previous_item', None)
-        item.pop('immediatelyAddableTypes', None)
-        item.pop('locallyAllowedTypes', None)
+        item.pop("@components", None)
+        item.pop("next_item", None)
+        item.pop("previous_item", None)
+        item.pop("immediatelyAddableTypes", None)
+        item.pop("locallyAllowedTypes", None)
 
         # 2. Remove all relationfields
         if HAS_AT and IBaseObject.providedBy(obj):
@@ -283,28 +366,30 @@ class ExportContent(BrowserView):
         elif HAS_DX and HAS_RELATIONS and IDexterityContent.providedBy(obj):
             for schema in iterSchemata(obj):
                 for name, field in getFields(schema).items():
-                    if IRelationChoice.providedBy(field) or IRelationList.providedBy(field):
+                    if IRelationChoice.providedBy(field) or IRelationList.providedBy(
+                        field
+                    ):
                         item.pop(name, None)
 
         # 3. Change default-fieldnames (AT to DX)
-        item = migrate_field(item, 'excludeFromNav', 'exclude_from_nav')
-        item = migrate_field(item, 'allowDiscussion', 'allow_discussion')
-        item = migrate_field(item, 'subject', 'subjects')
+        item = migrate_field(item, "excludeFromNav", "exclude_from_nav")
+        item = migrate_field(item, "allowDiscussion", "allow_discussion")
+        item = migrate_field(item, "subject", "subjects")
 
         # Some Date fields
-        item = migrate_field(item, 'expirationDate', 'expires')
-        item = migrate_field(item, 'effectiveDate', 'effective')
-        item = migrate_field(item, 'creation_date', 'created')
-        item = migrate_field(item, 'modification_date', 'modified')
+        item = migrate_field(item, "expirationDate", "expires")
+        item = migrate_field(item, "effectiveDate", "effective")
+        item = migrate_field(item, "creation_date", "created")
+        item = migrate_field(item, "modification_date", "modified")
 
         # Event fields
-        item = migrate_field(item, 'startDate', 'start')
-        item = migrate_field(item, 'endDate', 'end')
-        item = migrate_field(item, 'openEnd', 'open_end')
-        item = migrate_field(item, 'wholeDay', 'whole_day')
-        item = migrate_field(item, 'contactEmail', 'contact_email')
-        item = migrate_field(item, 'contactName', 'contact_name')
-        item = migrate_field(item, 'contactPhone', 'contact_phone')
+        item = migrate_field(item, "startDate", "start")
+        item = migrate_field(item, "endDate", "end")
+        item = migrate_field(item, "openEnd", "open_end")
+        item = migrate_field(item, "wholeDay", "whole_day")
+        item = migrate_field(item, "contactEmail", "contact_email")
+        item = migrate_field(item, "contactName", "contact_name")
+        item = migrate_field(item, "contactPhone", "contact_phone")
 
         # 4. Fix issue with AT Text fields
         # This is done in the ATTextFieldSerializer
@@ -316,10 +401,10 @@ class ExportContent(BrowserView):
         # TODO
 
         # 7. Fix view names on Folders and Collection
-        if self.safe_portal_type in ('collection', 'topic', 'folder'):
-            old_layout = item.get('layout', 'does_not_exist')
+        if self.safe_portal_type in ("collection", "topic", "folder"):
+            old_layout = item.get("layout", "does_not_exist")
             if old_layout in LISTING_VIEW_MAPPING:
-                item['layout'] = LISTING_VIEW_MAPPING[old_layout]
+                item["layout"] = LISTING_VIEW_MAPPING[old_layout]
 
         return item
 
@@ -329,10 +414,10 @@ class ExportContent(BrowserView):
         """
         obj_url = obj.absolute_url()
         parent_url = obj.__parent__.absolute_url()
-        if item['@id'] != obj_url:
-            item['@id'] = obj_url
-        if item['parent']['@id'] != parent_url:
-            item['parent']['@id'] = parent_url
+        if item["@id"] != obj_url:
+            item["@id"] = obj_url
+        if item["parent"]["@id"] != parent_url:
+            item["parent"]["@id"] = parent_url
         return item
 
 
