@@ -4,6 +4,7 @@ from OFS.interfaces import IOrderedContainer
 from operator import itemgetter
 from plone import api
 from plone.app.discussion.interfaces import IConversation
+from plone.app.textfield.value import RichTextValue
 from plone.restapi.interfaces import ISerializeToJson
 from plone.restapi.serializer.converters import json_compatible
 from plone.uuid.interfaces import IUUID
@@ -12,6 +13,21 @@ from Products.CMFCore.utils import getToolByName
 from Products.Five import BrowserView
 from zope.component import getMultiAdapter
 from zope.component import queryUtility
+from plone.app.portlets.interfaces import IPortletTypeInterface
+from plone.portlets.interfaces import IPortletAssignmentMapping
+from plone.portlets.interfaces import ILocalPortletAssignmentManager
+from plone.portlets.interfaces import IPortletAssignmentSettings
+from plone.portlets.interfaces import IPortletManager
+from plone.portlets.constants import (
+    USER_CATEGORY,
+    GROUP_CATEGORY,
+    CONTENT_TYPE_CATEGORY,
+    CONTEXT_CATEGORY,
+)
+from z3c.relationfield import RelationValue
+from zope.component import getUtilitiesFor
+from zope.component import queryMultiAdapter
+from zope.interface import providedBy
 
 import json
 import pkg_resources
@@ -41,7 +57,7 @@ class ExportRelations(BrowserView):
     """Export all relations"""
 
     def __call__(self, debug=False):
-        self.title = 'Export relations'
+        self.title = "Export relations"
         if not self.request.form.get("form.submitted", False):
             return self.index()
 
@@ -93,7 +109,9 @@ class ExportRelations(BrowserView):
                 portal_catalog = getToolByName(self.context, "portal_catalog")
                 for rel in relation_catalog.findRelations():
                     if rel.from_path and rel.to_path:
-                        from_brain = portal_catalog(path=dict(query=rel.from_path, depth=0))
+                        from_brain = portal_catalog(
+                            path=dict(query=rel.from_path, depth=0)
+                        )
                         to_brain = portal_catalog(path=dict(query=rel.to_path, depth=0))
                         if len(from_brain) > 0 and len(to_brain) > 0:
                             item = {
@@ -127,7 +145,7 @@ class ExportMembers(BrowserView):
     AUTO_ROLES = ["Authenticated"]
 
     def __call__(self):
-        self.title = 'Export members, groups and roles'
+        self.title = "Export members, groups and roles"
         if not self.request.form.get("form.submitted", False):
             return self.index()
 
@@ -224,7 +242,7 @@ class ExportTranslations(BrowserView):
     DROP_PATH = []
 
     def __call__(self):
-        self.title = 'Export translations'
+        self.title = "Export translations"
         if not self.request.form.get("form.submitted", False):
             return self.index()
 
@@ -238,8 +256,7 @@ class ExportTranslations(BrowserView):
         )
         return self.request.response.write(safe_bytes(data))
 
-
-    def all_translations(self):
+    def all_translations(self):  # noqa: C901
         results = []
 
         # Archetypes with LinguaPlone
@@ -253,10 +270,11 @@ class ExportTranslations(BrowserView):
 
             if HAS_LINGUAPLONE:
                 from Products.Archetypes.config import REFERENCE_CATALOG
+
                 # Get all data from the reference_catalog if it exists
                 reference_catalog = getToolByName(self.context, REFERENCE_CATALOG, None)
                 if reference_catalog is not None:
-                    for ref in reference_catalog(relationship='translationOf'):
+                    for ref in reference_catalog(relationship="translationOf"):
                         source = api.content.get(UID=ref.sourceUID)
                         if not source:
                             continue
@@ -264,7 +282,7 @@ class ExportTranslations(BrowserView):
                         translations = source.getTranslations()
                         for lang in translations:
                             if not lang:
-                                logger.info(u'Skip translation: {}'.format(lang))
+                                logger.info(u"Skip translation: {}".format(lang))
                                 continue
                             uuid = IUUID(translations[lang][0], None)
                             if uuid:
@@ -308,7 +326,7 @@ class ExportLocalRoles(BrowserView):
     """Export all local roles"""
 
     def __call__(self):
-        self.title = 'Export local roles'
+        self.title = "Export local roles"
         if not self.request.form.get("form.submitted", False):
             return self.index()
 
@@ -324,7 +342,6 @@ class ExportLocalRoles(BrowserView):
 
     def all_localroles(self):
         results = []
-        from Products.CMFPlone.utils import base_hasattr
 
         def get_localroles(obj, path):
             uid = IUUID(obj, None)
@@ -354,7 +371,7 @@ class ExportOrdering(BrowserView):
     """Export all local roles"""
 
     def __call__(self):
-        self.title = 'Export ordering'
+        self.title = "Export ordering"
         if not self.request.form.get("form.submitted", False):
             return self.index()
 
@@ -384,7 +401,9 @@ class ExportOrdering(BrowserView):
             return
 
         portal = api.portal.get()
-        portal.ZopeFindAndApply(portal, search_sub=True, apply_func=get_position_in_parent)
+        portal.ZopeFindAndApply(
+            portal, search_sub=True, apply_func=get_position_in_parent
+        )
         return sorted(results, key=itemgetter("order"))
 
 
@@ -392,7 +411,7 @@ class ExportDefaultPages(BrowserView):
     """Export all default_page settings."""
 
     def __call__(self):
-        self.title = 'Export default pages'
+        self.title = "Export default pages"
         if not self.request.form.get("form.submitted", False):
             return self.index()
 
@@ -425,7 +444,7 @@ class ExportDefaultPages(BrowserView):
 
 class ExportDiscussion(BrowserView):
     def __call__(self):
-        self.title = 'Export comments'
+        self.title = "Export comments"
         if not self.request.form.get("form.submitted", False):
             return self.index()
 
@@ -441,6 +460,7 @@ class ExportDiscussion(BrowserView):
 
     def all_discussions(self):
         results = []
+
         def get_discussion(obj, path):
             conversation = IConversation(obj, None)
             if not conversation:
@@ -456,6 +476,121 @@ class ExportDiscussion(BrowserView):
         return results
 
 
+class ExportPortlets(BrowserView):
+    def __call__(self):
+        self.title = "Export portlets"
+        if not self.request.form.get("form.submitted", False):
+            return self.index()
+
+        all_portlets = self.all_portlets()
+        data = json.dumps(all_portlets, indent=4)
+        filename = "portlets.json"
+        self.request.response.setHeader("Content-type", "application/json")
+        self.request.response.setHeader("content-length", len(data))
+        self.request.response.setHeader(
+            "Content-Disposition", 'attachment; filename="{0}"'.format(filename)
+        )
+        return self.request.response.write(safe_bytes(data))
+
+    def all_portlets(context=None):
+        results = []
+
+        def get_portlets(obj, path):
+            uid = IUUID(obj, None)
+            if not uid:
+                return
+            portlets = export_local_portlets(obj)
+            blacklist = export_portlets_blacklist(obj)
+            obj_results = {}
+            if portlets:
+                obj_results["portlets"] = portlets
+            if blacklist:
+                obj_results["blacklist_status"] = blacklist
+            if obj_results:
+                obj_results["uuid"] = uid
+                results.append(obj_results)
+            return
+
+        portal = api.portal.get()
+        portal.ZopeFindAndApply(portal, search_sub=True, apply_func=get_portlets)
+        return results
+
+
+def export_local_portlets(obj):
+    """Serialize portlets for one content object
+    Code mostly taken from https://github.com/plone/plone.restapi/pull/669
+    """
+    portlets_schemata = {
+        iface: name for name, iface in getUtilitiesFor(IPortletTypeInterface)
+    }
+    items = {}
+    for manager_name, manager in getUtilitiesFor(IPortletManager):
+        mapping = queryMultiAdapter((obj, manager), IPortletAssignmentMapping)
+        if mapping is None:
+            continue
+        mapping = mapping.__of__(obj)
+        for name, assignment in mapping.items():
+            portlet_type = None
+            schema = None
+            for schema in providedBy(assignment).flattened():
+                portlet_type = portlets_schemata.get(schema, None)
+                if portlet_type is not None:
+                    break
+            if portlet_type is None:
+                continue
+            assignment = assignment.__of__(mapping)
+            settings = IPortletAssignmentSettings(assignment)
+            if manager_name not in items:
+                items[manager_name] = []
+            values = {}
+            for name in schema.names():
+                value = getattr(assignment, name, None)
+                if isinstance(value, RelationValue):
+                    value = value.to_object.UID()
+                elif isinstance(value, RichTextValue):
+                    value = {
+                        "data": json_compatible(value.raw),
+                        "content-type": json_compatible(value.mimeType),
+                        "encoding": json_compatible(value.encoding),
+                    }
+                value = json_compatible(value)
+                values[name] = value
+            items[manager_name].append(
+                {
+                    "type": portlet_type,
+                    "visible": settings.get("visible", True),
+                    "assignment": values,
+                }
+            )
+    return items
+
+
+def export_portlets_blacklist(obj):
+    results = []
+    for manager_name, manager in getUtilitiesFor(IPortletManager):
+        assignable = queryMultiAdapter((obj, manager), ILocalPortletAssignmentManager)
+        if assignable is None:
+            continue
+        for category in (
+            USER_CATEGORY,
+            GROUP_CATEGORY,
+            CONTENT_TYPE_CATEGORY,
+            CONTEXT_CATEGORY,
+        ):
+            obj_results = {}
+            status = assignable.getBlacklistStatus(category)
+            if status is True:
+                obj_results["status"] = u"block"
+            elif status is False:
+                obj_results["status"] = u"show"
+
+            if obj_results:
+                obj_results["manager"] = manager_name
+                obj_results["category"] = category
+                results.append(obj_results)
+    return results
+
+
 def safe_bytes(value, encoding="utf-8"):
     """Convert text to bytes of the specified encoding."""
     if isinstance(value, six.text_type):
