@@ -1,9 +1,12 @@
 # -*- coding: utf-8 -*-
+from collective.exportimport import config
 from datetime import datetime
 from DateTime import DateTime
 from plone import api
 from plone.api.exc import InvalidParameterError
 from plone.i18n.normalizer.interfaces import IIDNormalizer
+from plone.namedfile.file import NamedBlobFile
+from plone.namedfile.file import NamedBlobImage
 from plone.restapi.interfaces import IDeserializeFromJson
 from Products.Five import BrowserView
 from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
@@ -22,11 +25,39 @@ import transaction
 
 try:
     from plone.app.querystring.upgrades import fix_select_all_existing_collections
+
     HAS_COLLECTION_FIX = True
 except ImportError:
     HAS_COLLECTION_FIX = False
 
 logger = logging.getLogger(__name__)
+BLOB_HOME = os.getenv("COLLECTIVE_EXPORTIMPORT_BLOB_HOME", "")
+
+
+def get_absolute_blob_path(obj, blob_path):
+    """Get absolute path to a blob.
+
+    If the BLOB_HOME variable is set, try to use this.
+
+    If the blob is not found there, try the blobstorage of the current ZODB.
+    The blob may be an export from a different Plone Site in the same database.
+    Or the blobstorage from the old 4.3 site may have been copied
+    or hard linked to the new 5.2 site.
+    """
+    if os.path.isabs(blob_path):
+        if os.path.isfile(blob_path):
+            return blob_path
+        return
+    if BLOB_HOME:
+        abs_path = os.path.join(BLOB_HOME, blob_path)
+        if os.path.isfile(abs_path):
+            return abs_path
+    # Try the blobstorage of the current ZODB.
+    db = obj._p_jar.db()
+    fshelper = db._storage.fshelper
+    abs_path = os.path.join(fshelper.base_dir, blob_path)
+    if os.path.isfile(abs_path):
+        return abs_path
 
 
 class ImportContent(BrowserView):
@@ -56,9 +87,8 @@ class ImportContent(BrowserView):
     # Example: {'which_price': 'normal'}
     DEFAULTS = {}
 
-    def __call__(self, jsonfile=None, portal_type=None, return_json=False, limit=None, server_file=None):
+    def __call__(self, jsonfile=None, return_json=False, limit=None, server_file=None):
         self.limit = limit
-        response = self.request.response
 
         if not self.request.form.get("form.submitted", False):
             return self.template()
@@ -71,7 +101,7 @@ class ImportContent(BrowserView):
             api.portal.show_message(
                 u"json file was uploaded, so the selected server file was ignored.",
                 request=self.request,
-                type="warn"
+                type="warn",
             )
             server_file = None
         if server_file and not jsonfile:
@@ -86,30 +116,18 @@ class ImportContent(BrowserView):
                         break
             else:
                 api.portal.show_message(
-                    u"File not found on server.",
-                    request=self.request,
-                    type="warn"
+                    u"File not found on server.", request=self.request, type="warn"
                 )
                 server_file = None
         if jsonfile:
             self.portal = api.portal.get()
-            self.portal_type = portal_type
             status = "success"
             try:
                 if isinstance(jsonfile, str):
-                    if not self.portal_type:
-                        raise RuntimeError(
-                            "portal_types required when passing a string"
-                        )
                     return_json = True
-                    data = ijson.items(jsonfile, 'item')
-                elif isinstance(jsonfile, FileUpload) or hasattr(jsonfile, 'read'):
-                    if not self.portal_type:
-                        if hasattr(jsonfile, 'filename'):
-                            self.portal_type = jsonfile.filename.split(".json")[0]
-                        elif hasattr(jsonfile, 'name'):
-                            self.portal_type = jsonfile.name.split("/")[-1].split(".json")[0]
-                    data = ijson.items(jsonfile, 'item')
+                    data = ijson.items(jsonfile, "item")
+                elif isinstance(jsonfile, FileUpload) or hasattr(jsonfile, "read"):
+                    data = ijson.items(jsonfile, "item")
                 else:
                     raise RuntimeError("Data is neither text, file nor upload.")
             except Exception as e:
@@ -151,6 +169,8 @@ class ImportContent(BrowserView):
             directory = os.path.join(impath, "import")
             listing.append(directory)
         listing.sort()
+        if config.CENTRAL_DIRECTORY:
+            listing.insert(0, config.CENTRAL_DIRECTORY)
         return listing
 
     @property
@@ -160,7 +180,11 @@ class ImportContent(BrowserView):
         for directory in self.import_paths:
             if not os.path.isdir(directory):
                 continue
-            listing += [f for f in os.listdir(directory) if f.endswith(".json")]
+            listing += [
+                f
+                for f in os.listdir(directory)
+                if f.endswith(".json") and f not in listing
+            ]
         listing.sort()
         return listing
 
@@ -169,22 +193,21 @@ class ImportContent(BrowserView):
         added = self.import_new_content(data)
         end = datetime.now()
         delta = end - start
-        msg = u"Imported {} {}".format(len(added), self.portal_type)
+        msg = u"Imported {} items".format(len(added))
         transaction.get().note(msg)
         transaction.commit()
         msg = u"{} in {} seconds".format(msg, delta.seconds)
         logger.info(msg)
         return msg
 
-    def import_new_content(self, data):
-        self.safe_portal_type = fix_portal_type(self.portal_type)
-        portal_workflow = api.portal.get_tool('portal_workflow')
+    def import_new_content(self, data):  # noqa: C901
+        portal_workflow = api.portal.get_tool("portal_workflow")
         added = []
 
-        if getattr(data, 'len', None):
-            logger.info(u"Importing {} {}".format(len(data), self.portal_type))
+        if getattr(data, "len", None):
+            logger.info(u"Importing {} items".format(len(data)))
         else:
-            logger.info(u"Importing {}".format(self.portal_type))
+            logger.info(u"Importing data")
         for index, item in enumerate(data, start=1):
             if self.limit and len(added) >= self.limit:
                 break
@@ -212,6 +235,7 @@ class ImportContent(BrowserView):
                 )
                 item["id"] = new_id
 
+            self.safe_portal_type = fix_portal_type(item["@type"])
             item = self.handle_broken(item)
             if not item:
                 continue
@@ -221,14 +245,20 @@ class ImportContent(BrowserView):
             item = self.global_dict_hook(item)
             if not item:
                 continue
+
+            # portal_type might change during a hook
+            self.safe_portal_type = fix_portal_type(item["@type"])
             item = self.custom_dict_hook(item)
             if not item:
                 continue
 
+            self.safe_portal_type = fix_portal_type(item["@type"])
             container = self.handle_container(item)
 
             if not container:
-                logger.info(u"No container found for {} {}".format(item["@type"], item["@id"]))
+                logger.info(
+                    u"No container found for {} {}".format(item["@type"], item["@id"])
+                )
                 continue
 
             # Speed up import by not using autogenerated ids for conflicts
@@ -242,7 +272,7 @@ class ImportContent(BrowserView):
                     )
                 )
 
-            factory_kwargs = item.get('factory_kwargs', {})
+            factory_kwargs = item.get("factory_kwargs", {})
             container.invokeFactory(item["@type"], item["id"], **factory_kwargs)
             new = container[item["id"]]
 
@@ -250,6 +280,12 @@ class ImportContent(BrowserView):
             deserializer = getMultiAdapter((new, self.request), IDeserializeFromJson)
             new = deserializer(validate_all=False, data=item)
 
+            # Blobs can be exported as only a path in the blob storage.
+            # It seems difficult to dynamically use a different deserializer,
+            # based on whether or not there is a blob_path somewhere in the item.
+            # So handle this case with a separate method.
+            self.import_blob_paths(new, item)
+
             self.global_obj_hook(new, item)
             self.custom_obj_hook(new, item)
 
@@ -268,13 +304,23 @@ class ImportContent(BrowserView):
             # These are reused and dropped in ResetModifiedAndCreatedDate
             modified = item.get("modified", item.get("modification_date", None))
             if modified:
-                modified_data = datetime.strptime(modified, "%Y-%m-%dT%H:%M:%S%z")
+                # Python 2 strptime does not know of %z timezone
+                try:
+                    modified_data = datetime.strptime(modified, "%Y-%m-%dT%H:%M:%S%z")
+                except ValueError:
+                    modified_data = datetime.strptime(
+                        modified[:-6], "%Y-%m-%dT%H:%M:%S"
+                    )
                 modification_date = DateTime(modified_data)
                 new.modification_date = modification_date
                 new.modification_date_migrated = modification_date
             created = item.get("created", item.get("creation_date", None))
             if created:
-                created_data = datetime.strptime(created, "%Y-%m-%dT%H:%M:%S%z")
+                # Python 2 strptime does not know of %z timezone
+                try:
+                    created_data = datetime.strptime(created, "%Y-%m-%dT%H:%M:%S%z")
+                except Exception:
+                    created_data = datetime.strptime(created[:-6], "%Y-%m-%dT%H:%M:%S")
                 creation_date = DateTime(created_data)
                 new.creation_date = creation_date
                 new.creation_date_migrated = creation_date
@@ -308,6 +354,41 @@ class ImportContent(BrowserView):
                 item[key] = self.DEFAULTS[key]
         return item
 
+    def import_blob_paths(self, new, item):
+        for key, value in item.items():
+            # Look for dictionaries with a blob_path key.
+            if not isinstance(value, dict):
+                continue
+            blob_path = value.get("blob_path")
+            if not blob_path:
+                continue
+            abs_blob_path = get_absolute_blob_path(new, blob_path)
+            if not abs_blob_path:
+                __traceback_info__ = item
+                raise ValueError("Blob path {} does not exist!".format(blob_path))
+
+            # Determine the class to use: file or image.
+            filename = value["filename"]
+            content_type = value["content-type"]
+            if key == "file":
+                klass = NamedBlobFile
+            elif key == "image":
+                klass = NamedBlobImage
+            elif content_type.startswith("image"):
+                klass = NamedBlobImage
+            else:
+                klass = NamedBlobFile
+
+            # Write the field.
+            with open(abs_blob_path, "rb") as myfile:
+                blobdata = myfile.read()
+            field_value = klass(
+                data=blobdata,
+                contentType=content_type,
+                filename=filename,
+            )
+            setattr(new, key, field_value)
+
     def global_dict_hook(self, item):
         """Overwrite this do general changes on the dict before deserializing.
 
@@ -351,11 +432,11 @@ class ImportContent(BrowserView):
 
         def handle_document_container(self, item):
             lang = item['language']['token'] if item['language'] else ''
-            base_path = self.CONTAINER[self.portal_type][item['language']['token']]
+            base_path = self.CONTAINER[item["@type"]][item['language']['token']]
             folder = api.content.get(path=base_path)
             if not folder:
                 raise RuntimeError(
-                    f'Target folder {base_path} for type {self.portal_type} is missing'
+                    f'Target folder {base_path} for type {item["@type"]} is missing'
                 )
             parent_url = item['parent']['@id']
             parent_path = '/'.join(parent_url.split('/')[5:])
@@ -405,13 +486,13 @@ class ImportContent(BrowserView):
         if self.request.get("import_to_current_folder", None):
             return self.context
 
-        container_path = self.CONTAINER.get(self.portal_type, None)
+        container_path = self.CONTAINER.get(item["@type"], None)
         if container_path:
             container = api.content.get(path=container_path)
             if not container:
                 raise RuntimeError(
                     u"Target folder {} for type {} is missing".format(
-                        container_path, self.portal_type
+                        container_path, item["@type"]
                     )
                 )
 
@@ -425,22 +506,83 @@ class ImportContent(BrowserView):
             return self.get_parent_as_container(item)
 
     def get_parent_as_container(self, item):
-        """The default is to generate a folder-structure exactly as the original"""
+        """The default is to generate a folder-structure exactly as the original.
+
+        There is some trickyness that probably only happens during local
+        development, and not in production sites.
+        Situation:
+
+        - localhost:8080/nl is a Dutch Plone Site
+        - localhost:8080/de is a German Plone Site in the same ZODB
+        - localhost:9999/fr is a French Plone Site a different ZODB
+
+        We export nl/folder/page.
+        The parent url (parent/@id) is http://localhost:8080/nl/folder
+        Parent path is then: ["", "nl", "folder].
+
+        1. We import it in the NL site.
+           We should recognize this, and not try to create
+           /nl/random-id/nl
+           which would fail with BadRequest at the reserved id 'nl'.
+           Note: the 'random-id' would be because of the empty string
+           at the start of the parent path.
+           Expected result: nl/folder/page-1234
+
+        2. We import it in the DE site.
+           This should *not* traverse to the NL site and create the content there.
+           I have seen this happen.
+           It should also *not* create a de/nl folder: an unwanted extra level.
+           Expected result: de/folder/page
+
+        3. We import it in the FR site.
+           It should *not* create a fr/nl folder: an unwanted extra level.
+           Expected result: fr/folder/page
+
+        """
         parent_url = unquote(item["parent"]["@id"])
         parent_path = urlparse(parent_url).path
+        # physical path is bytes in Zope 2 (not in Zope 4)
+        # so we need to encode parent_path before using plone.api.content.get
+        if isinstance(self.context.getPhysicalPath()[0], bytes):
+            parent_path = parent_path.encode("utf8")
         parent = api.content.get(path=parent_path)
         if parent:
-            return parent
-        else:
-            return self.create_container(item)
+            # Check that we did not traverse to content outside of the portal.
+            # Actually, we probably should not go outside the navigation root either,
+            # especially with multilingual, although most likely
+            # the context is the site root.
+            found_path = "/".join(parent.getPhysicalPath())
+            nav_path = "/".join(
+                api.portal.get_navigation_root(self.context).getPhysicalPath()
+            )
+            if found_path.startswith(nav_path):
+                return parent
+            logger.info(
+                "Ignoring existing container outside of navigation root: %s", found_path
+            )
+        return self.create_container(item)
 
     def create_container(self, item):
+        """Create container for item.
+
+        See remarks in get_parent_as_container for some corner cases.
+        """
         folder = self.context
         parent_url = unquote(item["parent"]["@id"])
-        parent_path = urlparse(parent_url).path
+        parent_url_parsed = urlparse(parent_url)
+        # Get the path part, split it, remove the always empty first element.
+        parent_path = parent_url_parsed.path.split("/")[1:]
+        if (
+            len(parent_url_parsed.netloc.split(":")) > 1
+            or parent_url_parsed.netloc == "nohost"
+        ):
+            # For example localhost:8080, or nohost when running tests.
+            # First element will then be a Plone Site id.
+            # Get rid of it.
+            parent_path = parent_path[1:]
 
         # create original structure for imported content
-        for element in parent_path.split("/"):
+        for element in parent_path:
             if element not in folder:
                 folder = api.content.create(
                     container=folder,
@@ -484,7 +626,7 @@ def fix_portal_type(portal_type):
 
 class ResetModifiedAndCreatedDate(BrowserView):
     def __call__(self):
-        self.title = 'Reset creation and modification date'
+        self.title = "Reset creation and modification date"
         self.help_text = """<p>Creation- and modification-dates are changed during import.
         This resets them to the original dates of the imported content.</p>"""
         if not self.request.form.get("form.submitted", False):
@@ -513,12 +655,13 @@ class ResetModifiedAndCreatedDate(BrowserView):
 
 class FixCollectionQueries(BrowserView):
     def __call__(self):
-        self.title = 'Fix collection queries'
+        self.title = "Fix collection queries"
         self.help_text = """<p>This fixes invalid collection-criteria that were imported from Plone 4 or 5.</p>"""
 
         if not HAS_COLLECTION_FIX:
             api.portal.show_message(
-                "plone.app.querystring.upgrades.fix_select_all_existing_collections is not available", self.request
+                "plone.app.querystring.upgrades.fix_select_all_existing_collections is not available",
+                self.request,
             )
             return self.index()
 
