@@ -5,17 +5,26 @@ from datetime import datetime
 from OFS.interfaces import IOrderedContainer
 from operator import itemgetter
 from plone import api
-from plone.app.discussion.browser.comments import CommentForm
 from plone.app.discussion.comment import Comment
 from plone.app.discussion.interfaces import IConversation
-from plone.restapi.deserializer import json_body
+from plone.app.portlets.interfaces import IPortletTypeInterface
+from plone.portlets.interfaces import IPortletAssignmentMapping
+from plone.portlets.interfaces import IPortletAssignmentSettings
+from plone.portlets.interfaces import ILocalPortletAssignmentManager
+from plone.portlets.interfaces import IPortletManager
 from Products.Five import BrowserView
+from six.moves.html_parser import HTMLParser
 from zope.annotation.interfaces import IAnnotations
+from zope.component import getUtility
+from zope.component import queryMultiAdapter
+from zope.component.interfaces import IFactory
+from zope.container.interfaces import INameChooser
 from ZPublisher.HTTPRequest import FileUpload
 
 import dateutil
 import json
 import logging
+import six
 import transaction
 
 try:
@@ -37,7 +46,7 @@ logger = logging.getLogger(__name__)
 DISCUSSION_ANNOTATION_KEY = "plone.app.discussion:conversation"
 
 
-if HAS_PAM:
+if HAS_PAM:  # noqa: C901
 
     class ImportTranslations(BrowserView):
         def __call__(self, jsonfile=None, return_json=False):
@@ -355,10 +364,16 @@ class ImportLocalRoles(BrowserView):
                 localroles = item["localroles"]
                 for userid in localroles:
                     obj.manage_setLocalRoles(userid=userid, roles=localroles[userid])
-                logger.info(u"Set roles on {}: {}".format(obj.absolute_url(), localroles))
+                logger.info(
+                    u"Set roles on {}: {}".format(obj.absolute_url(), localroles)
+                )
             if item.get("block"):
                 obj.__ac_local_roles_block__ = 1
-                logger.info(u"Diable acquisition of local roles on {}".format(obj.absolute_url()))
+                logger.info(
+                    u"Diable acquisition of local roles on {}".format(
+                        obj.absolute_url()
+                    )
+                )
             results += 1
         return results
 
@@ -448,7 +463,10 @@ class ImportDefaultPages(BrowserView):
             if not obj:
                 continue
             old = obj.getDefaultPage()
-            obj.setDefaultPage(item["default_page"])
+            if six.PY2:
+                obj.setDefaultPage(item["default_page"].encode("utf-8"))
+            else:
+                obj.setDefaultPage(item["default_page"])
             if old != obj.getDefaultPage():
                 results += 1
         return results
@@ -488,34 +506,41 @@ class ImportDiscussion(BrowserView):
 
     def import_data(self, data):
         results = 0
+        html = HTMLParser()
         for conversation_data in data:
             obj = api.content.get(UID=conversation_data["uuid"])
             if not obj:
                 continue
-            if not obj.restrictedTraverse('@@conversation_view').enabled():
+            if not obj.restrictedTraverse("@@conversation_view").enabled():
                 continue
             added = 0
             conversation = IConversation(obj)
 
-            for item in conversation_data['conversation']['items']:
+            for item in conversation_data["conversation"]["items"]:
 
-                if isinstance(item['text'], dict) and item['text'].get('data'):
-                    item['text'] = item['text']['data']
+                if isinstance(item["text"], dict) and item["text"].get("data"):
+                    item["text"] = item["text"]["data"]
 
                 comment = Comment()
-                comment_id = int(item['comment_id'])
+                comment_id = int(item["comment_id"])
                 comment.comment_id = comment_id
-                comment.creation_date = dateutil.parser.parse(item['creation_date'])
-                comment.modification_date = dateutil.parser.parse(item['modification_date'])
-                comment.author_name = item['author_name']
-                comment.author_username = item['author_username']
-                comment.creator = item['author_username']
-                comment.text = item['text']
+                comment.creation_date = dateutil.parser.parse(item["creation_date"])
+                comment.modification_date = dateutil.parser.parse(
+                    item["modification_date"]
+                )
+                comment.author_name = item["author_name"]
+                comment.author_username = item["author_username"]
+                comment.creator = item["author_username"]
+                comment.text = html.unescape(
+                    item["text"]
+                    .replace(u"\r<br />", u"\r\n")
+                    .replace(u"<br />", u"\r\n")
+                )
 
-                if item['user_notification']:
+                if item["user_notification"]:
                     comment.user_notification = True
-                if item.get('in_reply_to'):
-                    comment.in_reply_to = int(item['in_reply_to'])
+                if item.get("in_reply_to"):
+                    comment.in_reply_to = int(item["in_reply_to"])
 
                 conversation._comments[comment_id] = comment
                 comment.__parent__ = aq_base(conversation)
@@ -539,7 +564,107 @@ class ImportDiscussion(BrowserView):
                 if DISCUSSION_ANNOTATION_KEY not in annotions:
                     annotions[DISCUSSION_ANNOTATION_KEY] = aq_base(conversation)
                 added += 1
-            logger.info('Added {} comments to {}'.format(added, obj.absolute_url()))
+            logger.info("Added {} comments to {}".format(added, obj.absolute_url()))
             results += added
 
         return results
+
+
+class ImportPortlets(BrowserView):
+    """Import portlets"""
+
+    def __call__(self, jsonfile=None, return_json=False):
+        if jsonfile:
+            self.portal = api.portal.get()
+            status = "success"
+            try:
+                if isinstance(jsonfile, str):
+                    return_json = True
+                    data = json.loads(jsonfile)
+                elif isinstance(jsonfile, FileUpload):
+                    data = json.loads(jsonfile.read())
+                else:
+                    raise ("Data is neither text nor upload.")
+            except Exception as e:
+                status = "error"
+                logger.error(e)
+                api.portal.show_message(
+                    u"Failure while uploading: {}".format(e),
+                    request=self.request,
+                )
+            else:
+                portlets = self.import_portlets(data)
+                msg = u"Created {} portlets".format(portlets)
+                api.portal.show_message(msg, self.request)
+            if return_json:
+                msg = {"state": status, "msg": msg}
+                return json.dumps(msg)
+
+        return self.index()
+
+    def import_portlets(self, data):
+        results = 0
+        for item in data:
+            obj = api.content.get(UID=item["uuid"])
+            if not obj:
+                continue
+            registered_portlets = register_portlets(obj, item)
+            results += registered_portlets
+        return results
+
+
+def register_portlets(obj, item):
+    """Register portlets fror one object
+    Code adapted from plone.app.portlets.exportimport.portlets.PortletsXMLAdapter
+    Work in progress...
+    """
+    site = api.portal.get()
+    results = 0
+    for manager_name, portlets in item.get("portlets", {}).items():
+        manager = getUtility(IPortletManager, manager_name)
+        mapping = queryMultiAdapter((obj, manager), IPortletAssignmentMapping)
+        namechooser = INameChooser(mapping)
+
+        for portlet_data in portlets:
+            # 1. Create the assignment
+            assignment_data = portlet_data["assignment"]
+            portlet_type = portlet_data["type"]
+            portlet_factory = getUtility(IFactory, name=portlet_type)
+            assignment = portlet_factory()
+
+            name = namechooser.chooseName(None, assignment)
+            mapping[name] = assignment
+            logger.info("Added portlet {} to {}".format(name, obj.absolute_url()))
+
+            # aq-wrap it so that complex fields will work
+            assignment = assignment.__of__(site)
+
+            # set visibility setting
+            visible = portlet_data.get("visible")
+            if visible:
+                settings = IPortletAssignmentSettings(assignment)
+                settings["visible"] = visible
+
+            # 2. Apply portlet settings
+            portlet_interface = getUtility(IPortletTypeInterface, name=portlet_type)
+            for property_name, value in assignment_data.items():
+                field = portlet_interface.get(property_name, None)
+                if field is None:
+                    continue
+                field = field.bind(assignment)
+                field.set(assignment, value)
+
+            results += 1
+
+    for blacklist_status in item.get("blacklist_status", []):
+        status = blacklist_status["status"]
+        manager_name = blacklist_status["manager"]
+        category = blacklist_status["category"]
+        manager = getUtility(IPortletManager, manager_name)
+        assignable = queryMultiAdapter((obj, manager), ILocalPortletAssignmentManager)
+        if status.lower() == "block":
+            assignable.setBlacklistStatus(category, True)
+        elif status.lower() == "show":
+            assignable.setBlacklistStatus(category, False)
+
+    return results
