@@ -1,8 +1,9 @@
 # -*- coding: utf-8 -*-
-from hurry.filesize import size
 from collective.exportimport.interfaces import IBase64BlobsMarker
-from collective.exportimport.interfaces import IRawRichTextMarker
 from collective.exportimport.interfaces import IMigrationMarker
+from collective.exportimport.interfaces import IPathBlobsMarker
+from collective.exportimport.interfaces import IRawRichTextMarker
+from hurry.filesize import size
 from plone.app.textfield.interfaces import IRichText
 from plone.dexterity.interfaces import IDexterityContent
 from plone.namedfile.interfaces import INamedFileField
@@ -17,6 +18,7 @@ from zope.interface import implementer
 
 import base64
 import logging
+import os
 import pkg_resources
 
 try:
@@ -112,6 +114,15 @@ class RichttextFieldSerializerWithRawText(DefaultFieldSerializer):
             }
 
 
+def get_relative_blob_path(obj, full_path):
+    # Get blob path relative from the blobstorage root.
+    db = obj._p_jar.db()
+    base_dir = db._storage.fshelper.base_dir
+    if not full_path.startswith(base_dir):
+        return full_path
+    return full_path[len(base_dir):]
+
+
 if HAS_AT:
     from OFS.Image import Pdata
     from plone.app.blob.interfaces import IBlobField
@@ -125,7 +136,6 @@ if HAS_AT:
     from Products.Archetypes.interfaces.field import IImageField
     from Products.Archetypes.interfaces.field import ITextField
 
-
     @adapter(IImageField, IBaseObject, IBase64BlobsMarker)
     @implementer(IFieldSerializer)
     class ATImageFieldSerializer(ATDefaultFieldSerializer):
@@ -148,7 +158,6 @@ if HAS_AT:
             }
             return json_compatible(result)
 
-
     @adapter(IFileField, IBaseObject, IBase64BlobsMarker)
     @implementer(IFieldSerializer)
     class ATFileFieldSerializer(ATDefaultFieldSerializer):
@@ -176,7 +185,6 @@ if HAS_AT:
             }
             return json_compatible(result)
 
-
     @adapter(IBlobImageField, IBaseObject, IBase64BlobsMarker)
     @implementer(IFieldSerializer)
     class ATImageFieldSerializerWithBlobs(ATDefaultFieldSerializer):
@@ -199,7 +207,6 @@ if HAS_AT:
             }
             return json_compatible(result)
 
-
     @adapter(IBlobField, IBaseObject, IBase64BlobsMarker)
     @implementer(IFieldSerializer)
     class ATFileFieldSerializerWithBlobs(ATDefaultFieldSerializer):
@@ -226,6 +233,54 @@ if HAS_AT:
             }
             return json_compatible(result)
 
+    def get_at_blob_path(obj):
+        full_path = obj.getBlob().committed()
+        if not os.path.exists(full_path):
+            return
+        return get_relative_blob_path(obj, full_path)
+
+    @adapter(IBlobImageField, IBaseObject, IPathBlobsMarker)
+    @implementer(IFieldSerializer)
+    class ATImageFieldSerializerWithBlobPaths(ATDefaultFieldSerializer):
+        def __call__(self):
+            file_obj = self.field.get(self.context)
+            if not file_obj:
+                return None
+            blobfilepath = get_at_blob_path(file_obj)
+            if not blobfilepath:
+                logger.warning(
+                    "Blob file path of %s does not exist",
+                    self.context.absolute_url(),
+                )
+                return
+            result = {
+                "filename": self.field.getFilename(self.context),
+                "content-type": self.field.getContentType(self.context),
+                "blob_path": blobfilepath,
+            }
+            return json_compatible(result)
+
+    @adapter(IBlobField, IBaseObject, IPathBlobsMarker)
+    @implementer(IFieldSerializer)
+    class ATFileFieldSerializerWithBlobPaths(ATDefaultFieldSerializer):
+        def __call__(self):
+            file_obj = self.field.get(self.context)
+            if not file_obj:
+                return None
+
+            blobfilepath = get_at_blob_path(file_obj)
+            if not blobfilepath:
+                logger.warning(
+                    "Blob file path of %s does not exist",
+                    self.context.absolute_url(),
+                )
+                return
+            result = {
+                "filename": self.field.getFilename(self.context),
+                "content-type": self.field.getContentType(self.context),
+                "blob_path": blobfilepath,
+            }
+            return json_compatible(result)
 
     @adapter(ITextField, IBaseObject, IRawRichTextMarker)
     @implementer(IFieldSerializer)
@@ -259,17 +314,13 @@ if HAS_AT and HAS_PAC:
     from plone.restapi.serializer.atcontent import SerializeToJson
     from Products.ATContentTypes.interfaces.topic import IATTopic
 
-
     @implementer(ISerializeToJson)
     @adapter(IATTopic, IMigrationMarker)
     class SerializeTopicToJson(SerializeToJson):
-        """This uses the topic migration from p.a.contenttypes to turn Criteria into a Querystring.
-        """
+        """This uses the topic migration from p.a.contenttypes to turn Criteria into a Querystring."""
 
         def __call__(self, version=None, include_items=False):
-            topic_metadata = super(SerializeTopicToJson, self).__call__(
-                version=version
-            )
+            topic_metadata = super(SerializeTopicToJson, self).__call__(version=version)
 
             # migrate criteria
             formquery = []
@@ -280,12 +331,12 @@ if HAS_AT and HAS_PAC:
             criteria = self.context.listCriteria()
             for criterion in criteria:
                 type_ = criterion.__class__.__name__
-                if type_ == 'ATSortCriterion':
+                if type_ == "ATSortCriterion":
                     # Sort order and direction are now stored in the Collection.
                     self._collection_sort_reversed = criterion.getReversed()
                     self._collection_sort_on = criterion.Field()
                     logger.debug(
-                        'Sort on %r, reverse: %s.',
+                        "Sort on %r, reverse: %s.",
                         self._collection_sort_on,
                         self._collection_sort_reversed,
                     )
@@ -293,7 +344,7 @@ if HAS_AT and HAS_PAC:
 
                 converter = CONVERTERS.get(type_)
                 if converter is None:
-                    msg = 'Unsupported criterion {0}'.format(type_)
+                    msg = "Unsupported criterion {0}".format(type_)
                     logger.error(msg)
                     raise ValueError(msg)
                 try:
@@ -301,10 +352,70 @@ if HAS_AT and HAS_PAC:
                 except Exception as e:
                     logger.info(e)
 
-            topic_metadata['query'] = json_compatible(formquery)
+            topic_metadata["query"] = json_compatible(formquery)
 
             # migrate batch size
             if self.context.itemCount:
-                topic_metadata['b_size'] = self.context.itemCount
+                topic_metadata["b_size"] = self.context.itemCount
 
             return topic_metadata
+
+
+def get_dx_blob_path(obj):
+    full_path = obj._blob.committed()
+    if not os.path.exists(full_path):
+        return
+    return get_relative_blob_path(obj, full_path)
+
+
+@adapter(INamedFileField, IDexterityContent, IPathBlobsMarker)
+@implementer(IFieldSerializer)
+class FileFieldSerializerWithBlobPaths(DefaultFieldSerializer):
+
+    def __call__(self):
+        namedfile = self.field.get(self.context)
+        if namedfile is None:
+            return None
+
+        blobfilepath = get_dx_blob_path(namedfile)
+        if not blobfilepath:
+            logger.warning(
+                "Blob file path of %s does not exist",
+                self.context.absolute_url(),
+            )
+            return
+        result = {
+            "filename": namedfile.filename,
+            "content-type": namedfile.contentType,
+            "size": namedfile.getSize(),
+            "blob_path": blobfilepath,
+        }
+        return json_compatible(result)
+
+
+@adapter(INamedImageField, IDexterityContent, IPathBlobsMarker)
+@implementer(IFieldSerializer)
+class ImageFieldSerializerWithBlobPaths(DefaultFieldSerializer):
+
+    def __call__(self):
+        image = self.field.get(self.context)
+        if image is None:
+            return None
+
+        blobfilepath = get_dx_blob_path(image)
+        if not blobfilepath:
+            logger.warning(
+                "Blob file path of %s does not exist",
+                self.context.absolute_url(),
+            )
+            return
+        width, height = image.getImageSize()
+        result = {
+            "filename": image.filename,
+            "content-type": image.contentType,
+            "size": image.getSize(),
+            "width": width,
+            "height": height,
+            "blob_path": blobfilepath,
+        }
+        return json_compatible(result)
